@@ -6,7 +6,7 @@ use strict;
 use warnings;
 use Encode;
 
-our $VERSION = '0.94';
+our $VERSION = '0.96';
 $VERSION = eval $VERSION;    ## no critic (BuiltinFunctions::ProhibitStringyEval)
 
 BEGIN {
@@ -25,7 +25,7 @@ BEGIN {
         require threads::shared;
 
         # Hack around YET ANOTHER threads::shared bug.  It would
-        # occassionally forget the contents of the variable when sharing it.
+        # occasionally forget the contents of the variable when sharing it.
         # So we first copy the data, then share, then put our copy back.
         *share = sub (\[$@%]) {
             my $type = ref $_[0];
@@ -100,25 +100,35 @@ sub child {
         $self->croak("You already have a child named ($self->{Child_Name}) running");
     }
 
+    my $parent_in_todo = $self->in_todo;
+
+    # Clear $TODO for the child.
+    my $orig_TODO = $self->find_TODO(undef, 1, undef);
+
     my $child = bless {}, ref $self;
     $child->reset;
 
     # Add to our indentation
     $child->_indent( $self->_indent . '    ' );
+    
     $child->{$_} = $self->{$_} foreach qw{Out_FH Todo_FH Fail_FH};
+    if ($parent_in_todo) {
+        $child->{Fail_FH} = $self->{Todo_FH};
+    }
 
     # This will be reset in finalize. We do this here lest one child failure
     # cause all children to fail.
     $child->{Child_Error} = $?;
     $?                    = 0;
     $child->{Parent}      = $self;
+    $child->{Parent_TODO} = $orig_TODO;
     $child->{Name}        = $name || "Child of " . $self->name;
     $self->{Child_Name}   = $child->name;
     return $child;
 }
 
 
-#line 202
+#line 212
 
 sub subtest {
     my $self = shift;
@@ -130,27 +140,50 @@ sub subtest {
 
     # Turn the child into the parent so anyone who has stored a copy of
     # the Test::Builder singleton will get the child.
-    my $child = $self->child($name);
-    my %parent = %$self;
-    %$self = %$child;
+    my($error, $child, %parent);
+    {
+        # child() calls reset() which sets $Level to 1, so we localize
+        # $Level first to limit the scope of the reset to the subtest.
+        local $Test::Builder::Level = $Test::Builder::Level + 1;
 
-    my $error;
-    if( !eval { $subtests->(); 1 } ) {
-        $error = $@;
+        $child  = $self->child($name);
+        %parent = %$self;
+        %$self  = %$child;
+
+        my $run_the_subtests = sub {
+            $subtests->();
+            $self->done_testing unless $self->_plan_handled;
+            1;
+        };
+
+        if( !eval { $run_the_subtests->() } ) {
+            $error = $@;
+        }
     }
 
     # Restore the parent and the copied child.
     %$child = %$self;
     %$self = %parent;
 
+    # Restore the parent's $TODO
+    $self->find_TODO(undef, 1, $child->{Parent_TODO});
+
     # Die *after* we restore the parent.
     die $error if $error and !eval { $error->isa('Test::Builder::Exception') };
 
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     return $child->finalize;
 }
 
+#line 282
 
-#line 251
+sub _plan_handled {
+    my $self = shift;
+    return $self->{Have_Plan} || $self->{No_Plan} || $self->{Skip_All};
+}
+
+
+#line 307
 
 sub finalize {
     my $self = shift;
@@ -164,6 +197,7 @@ sub finalize {
     # XXX This will only be necessary for TAP envelopes (we think)
     #$self->_print( $self->is_passing ? "PASS\n" : "FAIL\n" );
 
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my $ok = 1;
     $self->parent->{Child_Name} = undef;
     if ( $self->{Skip_All} ) {
@@ -191,17 +225,17 @@ sub _indent      {
     return $self->{Indent};
 }
 
-#line 301
+#line 358
 
 sub parent { shift->{Parent} }
 
-#line 313
+#line 370
 
 sub name { shift->{Name} }
 
 sub DESTROY {
     my $self = shift;
-    if ( $self->parent ) {
+    if ( $self->parent and $$ == $self->{Original_Pid} ) {
         my $name = $self->name;
         $self->diag(<<"FAIL");
 Child ($name) exited without calling finalize()
@@ -211,7 +245,7 @@ FAIL
     }
 }
 
-#line 337
+#line 394
 
 our $Level;
 
@@ -228,6 +262,7 @@ sub reset {    ## no critic (Subroutines::ProhibitBuiltinHomonyms)
     $self->{Have_Plan}    = 0;
     $self->{No_Plan}      = 0;
     $self->{Have_Output_Plan} = 0;
+    $self->{Done_Testing} = 0;
 
     $self->{Original_Pid} = $$;
     $self->{Child_Name}   = undef;
@@ -257,7 +292,7 @@ sub reset {    ## no critic (Subroutines::ProhibitBuiltinHomonyms)
     return;
 }
 
-#line 415
+#line 473
 
 my %plan_cmds = (
     no_plan     => \&no_plan,
@@ -304,8 +339,7 @@ sub _plan_tests {
     return;
 }
 
-
-#line 471
+#line 528
 
 sub expected_tests {
     my $self = shift;
@@ -323,7 +357,7 @@ sub expected_tests {
     return $self->{Expected_Tests};
 }
 
-#line 495
+#line 552
 
 sub no_plan {
     my($self, $arg) = @_;
@@ -336,8 +370,7 @@ sub no_plan {
     return 1;
 }
 
-
-#line 529
+#line 585
 
 sub _output_plan {
     my($self, $max, $directive, $reason) = @_;
@@ -355,7 +388,8 @@ sub _output_plan {
     return;
 }
 
-#line 580
+
+#line 637
 
 sub done_testing {
     my($self, $num_tests) = @_;
@@ -398,7 +432,7 @@ sub done_testing {
 }
 
 
-#line 631
+#line 688
 
 sub has_plan {
     my $self = shift;
@@ -408,7 +442,7 @@ sub has_plan {
     return(undef);
 }
 
-#line 648
+#line 705
 
 sub skip_all {
     my( $self, $reason ) = @_;
@@ -422,7 +456,7 @@ sub skip_all {
     exit(0);
 }
 
-#line 673
+#line 730
 
 sub exported_to {
     my( $self, $pack ) = @_;
@@ -433,7 +467,7 @@ sub exported_to {
     return $self->{Exported_To};
 }
 
-#line 703
+#line 760
 
 sub ok {
     my( $self, $test, $name ) = @_;
@@ -593,14 +627,12 @@ sub _is_dualvar {
     return $numval != 0 and $numval ne $val ? 1 : 0;
 }
 
-#line 877
+#line 934
 
 sub is_eq {
     my( $self, $got, $expect, $name ) = @_;
     local $Level = $Level + 1;
 
-    $self->_unoverload_str( \$got, \$expect );
-
     if( !defined $got || !defined $expect ) {
         # undef only matches undef and nothing else
         my $test = !defined $got && !defined $expect;
@@ -617,8 +649,6 @@ sub is_num {
     my( $self, $got, $expect, $name ) = @_;
     local $Level = $Level + 1;
 
-    $self->_unoverload_num( \$got, \$expect );
-
     if( !defined $got || !defined $expect ) {
         # undef only matches undef and nothing else
         my $test = !defined $got && !defined $expect;
@@ -676,7 +706,7 @@ sub _isnt_diag {
 DIAGNOSTIC
 }
 
-#line 974
+#line 1027
 
 sub isnt_eq {
     my( $self, $got, $dont_expect, $name ) = @_;
@@ -710,7 +740,7 @@ sub isnt_num {
     return $self->cmp_ok( $got, '!=', $dont_expect, $name );
 }
 
-#line 1023
+#line 1076
 
 sub like {
     my( $self, $this, $regex, $name ) = @_;
@@ -726,7 +756,7 @@ sub unlike {
     return $self->_regex_ok( $this, $regex, '!~', $name );
 }
 
-#line 1047
+#line 1100
 
 my %numeric_cmps = map { ( $_, 1 ) } ( "<", "<=", ">", ">=", "==", "!=", "<=>" );
 
@@ -742,8 +772,9 @@ sub cmp_ok {
 
         my($pack, $file, $line) = $self->caller();
 
+        # This is so that warnings come out at the caller's level
         $test = eval qq[
-#line 1 "cmp_ok [from $file line $line]"
+#line $line "(eval in cmp_ok) $file"
 \$got $type \$expect;
 ];
         $error = $@;
@@ -806,7 +837,7 @@ sub _caller_context {
     return $code;
 }
 
-#line 1146
+#line 1200
 
 sub BAIL_OUT {
     my( $self, $reason ) = @_;
@@ -816,14 +847,14 @@ sub BAIL_OUT {
     exit 255;
 }
 
-#line 1159
+#line 1213
 
 {
     no warnings 'once';
     *BAILOUT = \&BAIL_OUT;
 }
 
-#line 1173
+#line 1227
 
 sub skip {
     my( $self, $why ) = @_;
@@ -854,7 +885,7 @@ sub skip {
     return 1;
 }
 
-#line 1214
+#line 1268
 
 sub todo_skip {
     my( $self, $why ) = @_;
@@ -882,7 +913,7 @@ sub todo_skip {
     return 1;
 }
 
-#line 1294
+#line 1348
 
 sub maybe_regex {
     my( $self, $regex ) = @_;
@@ -962,7 +993,7 @@ DIAGNOSTIC
 # I'm not ready to publish this.  It doesn't deal with array return
 # values from the code or context.
 
-#line 1390
+#line 1444
 
 sub _try {
     my( $self, $code, %opts ) = @_;
@@ -982,7 +1013,7 @@ sub _try {
     return wantarray ? ( $return, $error ) : $return;
 }
 
-#line 1419
+#line 1473
 
 sub is_fh {
     my $self     = shift;
@@ -996,7 +1027,7 @@ sub is_fh {
            eval { tied($maybe_fh)->can('TIEHANDLE') };
 }
 
-#line 1462
+#line 1516
 
 sub level {
     my( $self, $level ) = @_;
@@ -1007,7 +1038,7 @@ sub level {
     return $Level;
 }
 
-#line 1494
+#line 1548
 
 sub use_numbers {
     my( $self, $use_nums ) = @_;
@@ -1018,7 +1049,7 @@ sub use_numbers {
     return $self->{Use_Nums};
 }
 
-#line 1527
+#line 1581
 
 foreach my $attribute (qw(No_Header No_Ending No_Diag)) {
     my $method = lc $attribute;
@@ -1036,7 +1067,7 @@ foreach my $attribute (qw(No_Header No_Ending No_Diag)) {
     *{ __PACKAGE__ . '::' . $method } = $code;
 }
 
-#line 1580
+#line 1634
 
 sub diag {
     my $self = shift;
@@ -1044,7 +1075,7 @@ sub diag {
     $self->_print_comment( $self->_diag_fh, @_ );
 }
 
-#line 1595
+#line 1649
 
 sub note {
     my $self = shift;
@@ -1081,7 +1112,7 @@ sub _print_comment {
     return 0;
 }
 
-#line 1645
+#line 1699
 
 sub explain {
     my $self = shift;
@@ -1100,7 +1131,7 @@ sub explain {
     } @_;
 }
 
-#line 1674
+#line 1728
 
 sub _print {
     my $self = shift;
@@ -1115,24 +1146,25 @@ sub _print_to_fh {
     return if $^C;
 
     my $msg = join '', @msgs;
+    my $indent = $self->_indent;
 
     local( $\, $", $, ) = ( undef, ' ', '' );
 
     # Escape each line after the first with a # so we don't
     # confuse Test::Harness.
-    $msg =~ s{\n(?!\z)}{\n# }sg;
+    $msg =~ s{\n(?!\z)}{\n$indent# }sg;
 
     # Stick a newline on the end if it needs it.
     $msg .= "\n" unless $msg =~ /\n\z/;
 
     if ( Encode::is_utf8($msg) ) {
-        return print $fh $self->_indent, Encode::encode_utf8($msg);
+        return print $fh $indent, Encode::encode_utf8($msg);
     } else {
-        return print $fh $self->_indent, $msg;
+        return print $fh $indent, $msg;
     }
 }
 
-#line 1737
+#line 1792
 
 sub output {
     my( $self, $fh ) = @_;
@@ -1251,7 +1283,7 @@ sub _copy_io_layers {
     return;
 }
 
-#line 1862
+#line 1917
 
 sub reset_outputs {
     my $self = shift;
@@ -1263,7 +1295,7 @@ sub reset_outputs {
     return;
 }
 
-#line 1888
+#line 1943
 
 sub _message_at_caller {
     my $self = shift;
@@ -1284,7 +1316,7 @@ sub croak {
 }
 
 
-#line 1928
+#line 1983
 
 sub current_test {
     my( $self, $num ) = @_;
@@ -1317,7 +1349,7 @@ sub current_test {
     return $self->{Curr_Test};
 }
 
-#line 1976
+#line 2031
 
 sub is_passing {
     my $self = shift;
@@ -1330,7 +1362,7 @@ sub is_passing {
 }
 
 
-#line 1998
+#line 2053
 
 sub summary {
     my($self) = shift;
@@ -1338,14 +1370,14 @@ sub summary {
     return map { $_->{'ok'} } @{ $self->{Test_Results} };
 }
 
-#line 2053
+#line 2108
 
 sub details {
     my $self = shift;
     return @{ $self->{Test_Results} };
 }
 
-#line 2082
+#line 2137
 
 sub todo {
     my( $self, $pack ) = @_;
@@ -1359,19 +1391,21 @@ sub todo {
     return '';
 }
 
-#line 2104
+#line 2164
 
 sub find_TODO {
-    my( $self, $pack ) = @_;
+    my( $self, $pack, $set, $new_value ) = @_;
 
     $pack = $pack || $self->caller(1) || $self->exported_to;
     return unless $pack;
 
     no strict 'refs';    ## no critic
-    return ${ $pack . '::TODO' };
+    my $old_value = ${ $pack . '::TODO' };
+    $set and ${ $pack . '::TODO' } = $new_value;
+    return $old_value;
 }
 
-#line 2122
+#line 2184
 
 sub in_todo {
     my $self = shift;
@@ -1380,7 +1414,7 @@ sub in_todo {
     return( defined $self->{Todo} || $self->find_TODO ) ? 1 : 0;
 }
 
-#line 2172
+#line 2234
 
 sub todo_start {
     my $self = shift;
@@ -1395,7 +1429,7 @@ sub todo_start {
     return;
 }
 
-#line 2194
+#line 2256
 
 sub todo_end {
     my $self = shift;
@@ -1416,7 +1450,7 @@ sub todo_end {
     return;
 }
 
-#line 2227
+#line 2289
 
 sub caller {    ## no critic (Subroutines::ProhibitBuiltinHomonyms)
     my( $self, $height ) = @_;
@@ -1431,9 +1465,9 @@ sub caller {    ## no critic (Subroutines::ProhibitBuiltinHomonyms)
     return wantarray ? @caller : $caller[0];
 }
 
-#line 2244
+#line 2306
 
-#line 2258
+#line 2320
 
 #'#
 sub _sanity_check {
@@ -1446,7 +1480,7 @@ sub _sanity_check {
     return;
 }
 
-#line 2279
+#line 2341
 
 sub _whoa {
     my( $self, $check, $desc ) = @_;
@@ -1461,7 +1495,7 @@ WHOA
     return;
 }
 
-#line 2303
+#line 2365
 
 sub _my_exit {
     $? = $_[0];    ## no critic (Variables::RequireLocalizedPunctuationVars)
@@ -1469,7 +1503,7 @@ sub _my_exit {
     return 1;
 }
 
-#line 2315
+#line 2377
 
 sub _ending {
     my $self = shift;
@@ -1588,7 +1622,7 @@ END {
     $Test->_ending if defined $Test;
 }
 
-#line 2503
+#line 2565
 
 1;
 
